import sys,time
from os import environ
from os import getcwd
import string
sys.path.append(environ["PYTHON_MODULE_PATH"])
sys.path.append(environ["SWIG_LIB_INSTALL_DIR"])
import os

global Lx, Ly, Lz
global T, nOrderFlip, nOrderContact, time
global cd, margin
global LamV, LamS, tV, tS

#VOLUME/SURFACE PARAMETERS:
cd=7          #Cell's diameter
tV=cd*cd      #Target Volume
LamV=5        #Lambda Volume
tS=0          #Target Surface
LamS=0        #Lambda Surface
#
#PARAMETERS:
T=15             #Temperature
nOrderFlip=2     #Distance of interaction
nOrderContact=4  #Distance of interaction
time=1500        #time of simulation
margin=2*cd      #margin
#
#SPACE PARAMETERS:
Lx=10*cd+2*margin   #horizontal width
Ly=10*cd+2*margin   #vertical height
Lz=1                #3rd dimension (1=>2D)

def configureSimulation(sim):
   import CompuCellSetup
   from XMLUtils import ElementCC3D
   cc3d=ElementCC3D("CompuCell3D")

   md=cc3d.ElementCC3D("Metadata")
   md.ElementCC3D("VirtualProcessingUnits",{"ThreadsPerVPU":2},2)
   md.ElementCC3D("DebugOutputFrequency",{},0)

   potts=cc3d.ElementCC3D("Potts")
   potts.ElementCC3D("Dimensions",{"x":Lx,"y":Ly,"z":Lz})
   potts.ElementCC3D("Steps",{},time)
   potts.ElementCC3D("Temperature",{},T)
   potts.ElementCC3D("NeighborOrder",{},nOrderFlip)
   potts.ElementCC3D("Boundary_x",{},"Periodic")
   potts.ElementCC3D("Boundary_y",{},"Periodic")
   
   cellType=cc3d.ElementCC3D("Plugin",{"Name":"CellType"})
   cellType.ElementCC3D("CellType", {"TypeName":"Medium","TypeId":0})
   cellType.ElementCC3D("CellType", {"TypeName":"Cells", "TypeId":1})
   cellType.ElementCC3D("CellType", {"TypeName":"Wall",  "TypeId":2})
   
   PlaySet =cc3d.ElementCC3D("Plugin",{"Name":"PlayerSettings"})
   PlaySet.ElementCC3D("Cell", {"Type":0, "Color":"#000000"}) #black
   PlaySet.ElementCC3D("Cell", {"Type":1, "Color":"#FF0000"}) #red
   PlaySet.ElementCC3D("Cell", {"Type":2, "Color":"#00FF00"}) #green
   PlaySet.ElementCC3D("VisualControl", {"ScreenshotFrequency":time, "ScreenUpdateFrequency":50})

   contact=cc3d.ElementCC3D("Plugin",{"Name":"Contact"})
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Medium"},0)
   contact.ElementCC3D("Energy", {"Type1":"Medium", "Type2": "Cells"},10)
   contact.ElementCC3D("Energy", {"Type1":"Cells",  "Type2": "Cells"},10)
   contact.ElementCC3D("NeighborOrder",{},nOrderContact)
   
   volume =cc3d.ElementCC3D("Plugin",{"Name":"Volume"})
   volume.ElementCC3D("TargetVolume",{},tV)
   volume.ElementCC3D("LambdaVolume",{},LamV)
 
   surface =cc3d.ElementCC3D("Plugin",{"Name":"Surface"})
   surface.ElementCC3D("TargetSurface",{},tS)
   surface.ElementCC3D("LambdaSurface",{},LamS)

   comtpd = cc3d.ElementCC3D("Plugin",{"Name":"CenterOfMass"})
   
   #CELLS INITIAL CONFIGURATION:
   uipd = cc3d.ElementCC3D("Steppable",{"Type":"UniformInitializer"})
   region = uipd.ElementCC3D("Region") #Cells
   region.ElementCC3D("BoxMin",{"x":margin,    "y":margin,    "z":0})
   region.ElementCC3D("BoxMax",{"x":Lx-margin, "y":Ly-margin, "z":1})
   region.ElementCC3D("Types",{}, "Cells")
   region.ElementCC3D("Width",{}, cd)
   
   bwpd=cc3d.ElementCC3D("Steppable",{"Type":"BoxWatcher"})
   bwpd.ElementCC3D("XMargin",{},2)
   bwpd.ElementCC3D("YMargin",{},2)
   bwpd.ElementCC3D("ZMargin",{},2)
   
   CompuCellSetup.setSimulationXMLDescription(cc3d)

import CompuCellSetup

#CompuCellSetup.simulationPaths.simulationPythonScriptName = "something :)"
sim,simthread = CompuCellSetup.getCoreSimulationObjects()

configureSimulation(sim)

import CompuCell
CompuCellSetup.initializeSimulationObjects(sim,simthread)

pyAttributeAdder,dictAdder=CompuCellSetup.attachDictionaryToCells(sim)

#Add Python steppables here
steppableRegistry=CompuCellSetup.getSteppableRegistry()

changeWatcherRegistry=CompuCellSetup.getChangeWatcherRegistry(sim)
stepperRegistry=CompuCellSetup.getStepperRegistry(sim)

from DiffusionCalculatorSteppables import DiffusionCalc
diffusionCalc=DiffusionCalc(_simulator=sim,_frequency=1,_time=time)
steppableRegistry.registerSteppable(diffusionCalc)

#Create extra player fields here or add attributes
dim=sim.getPotts().getCellFieldG().getDim()

CompuCellSetup.mainLoop(sim,simthread,steppableRegistry)
#sys.exit()
